@@ -1,10 +1,33 @@
-#ifndef _YAHTTP_UTILITY_HPP
-#define _YAHTTP_UTILITY_HPP 1
-
+#pragma once
 namespace YaHTTP {
   static const char *MONTHS[] = {0,"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec",0}; //<! List of months 
   static const char *DAYS[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat",0}; //<! List of days
 
+  bool isspace(char c);
+  bool isspace(char c, const std::locale& loc);
+  bool isxdigit(char c);
+  bool isxdigit(char c, const std::locale& loc);
+  bool isdigit(char c);
+  bool isdigit(char c, const std::locale& loc);
+  bool isalnum(char c);
+  bool isalnum(char c, const std::locale& loc);
+
+  /*! Case-Insensitive NULL safe comparator for string maps */
+  struct ASCIICINullSafeComparator {
+    bool operator() (const std::string& lhs, const std::string& rhs) const {
+      int v;
+      std::string::const_iterator lhi = lhs.begin();
+      std::string::const_iterator rhi = rhs.begin();
+      for(;lhi != lhs.end() && rhi != rhs.end(); lhi++, rhi++)
+        if ((v = ::tolower(*lhi) - ::tolower(*rhi)) != 0) return v<0; 
+      if (lhi == lhs.end() && rhi != rhs.end()) return true;
+      if (lhi != lhs.end() && rhi == rhs.end()) return false;
+      return false; // they are equal
+    }
+  };
+
+  typedef std::map<std::string,std::string,ASCIICINullSafeComparator> strstr_map_t; //<! String to String map
+
   /*! Represents a date/time with utc offset */
   class DateTime {
   public:
@@ -93,12 +116,12 @@ namespace YaHTTP {
      }; //<! parses date from struct tm 
 
      void validate() const {
-       if (wday < 0 || wday > 6) throw "Invalid date";
-       if (month < 1 || month > 12) throw "Invalid date";
-       if (year < 0) throw "Invalid date";
+       if (wday < 0 || wday > 6) throw std::range_error("Invalid date");
+       if (month < 1 || month > 12) throw std::range_error("Invalid date");
+       if (year < 0) throw std::range_error("Invalid date");
        if (hours < 0 || hours > 23 ||
            minutes < 0 || minutes > 59 ||
-           seconds < 0 || seconds > 60) throw "Invalid date";
+           seconds < 0 || seconds > 60) throw std::range_error("Invalid date");
      }; //<! make sure we are within ranges (not a *REAL* validation, just range check)
 
      std::string rfc_str() const {
@@ -137,33 +160,38 @@ namespace YaHTTP {
 	if ( (ptr = strptime(rfc822_date.c_str(), "%a, %d %b %Y %T", &tm)) != NULL) {
           int sign;
   	  // parse the timezone parameter
-          while(*ptr && ::isspace(*ptr)) ptr++;
+          while(*ptr && YaHTTP::isspace(*ptr)) ptr++;
           if (*ptr == '+') sign = 0;
           else if (*ptr == '-') sign = -1;
-          else throw "Unparseable date";
+          else throw YaHTTP::ParseError("Unparseable date");
           ptr++;
           utc_offset = ::atoi(ptr) * sign;
-          while(*ptr && ::isdigit(*ptr)) ptr++;
+          while(*ptr != '\0' && YaHTTP::isdigit(*ptr)) ptr++;
 #endif
-          while(*ptr && ::isspace(*ptr)) ptr++;
-          if (*ptr) throw "Unparseable date"; // must be final.
+          while(*ptr != '\0' && YaHTTP::isspace(*ptr)) ptr++;
+          if (*ptr != '\0') throw YaHTTP::ParseError("Unparseable date"); // must be final.
           fromTm(&tm);
        } else {
-          throw "Unparseable date";
+          throw YaHTTP::ParseError("Unparseable date");
        }
      }; //<! parses RFC-822 date
 
      void parseCookie(const std::string &cookie_date) {
        struct tm tm;
        const char *ptr;
-       if ( (ptr = strptime(cookie_date.c_str(), "%d-%b-%Y %T", &tm)) != NULL) {
-          while(*ptr && ( ::isspace(*ptr) || ::isalnum(*ptr) )) ptr++;
-          std::cerr << ptr << std::endl;
-          if (*ptr) throw "Unparseable date (non-final)"; // must be final.
+       if ( (ptr = strptime(cookie_date.c_str(), "%d-%b-%Y %T", &tm)) != NULL
+#ifdef HAVE_TM_GMTOFF
+          || (ptr = strptime(cookie_date.c_str(), "%d-%b-%Y %T %z", &tm)) != NULL
+          || (ptr = strptime(cookie_date.c_str(), "%a, %d-%b-%Y %T %Z", &tm)) != NULL
+#endif
+          ) {
+          while(*ptr != '\0' && ( YaHTTP::isspace(*ptr) || YaHTTP::isalnum(*ptr) )) ptr++;
+          if (*ptr != '\0') throw YaHTTP::ParseError("Unparseable date (non-final)"); // must be final.
           fromTm(&tm);
           this->utc_offset = 0;
        } else {
-          throw "Unparseable date (did not match pattern cookie)";
+          std::cout << cookie_date << std::endl;
+          throw YaHTTP::ParseError("Unparseable date (did not match pattern cookie)");
        }
      }; //<! parses HTTP Cookie date
 
@@ -221,7 +249,7 @@ namespace YaHTTP {
       char repl[3];
       size_t pos;
       for(std::string::iterator iter = result.begin(); iter != result.end(); iter++) {
-        if (!std::isalnum(*iter) && (!asUrl || skip.find(*iter) == std::string::npos)) {
+        if (!YaHTTP::isalnum(*iter) && (!asUrl || skip.find(*iter) == std::string::npos)) {
           // replace with different thing
           pos = std::distance(result.begin(), iter);
           ::snprintf(repl,3,"%02x", static_cast<unsigned char>(*iter));
@@ -240,7 +268,7 @@ namespace YaHTTP {
       std::ostringstream result;
       std::string skip = "+-.,&;_#%[]?/@(){}=";
       for(std::vector<unsigned char>::iterator iter = vec.begin(); iter != vec.end(); iter++) {
-        if (!std::isalnum((char)*iter) && (!asUrl || skip.find((char)*iter) == std::string::npos)) {
+        if (!YaHTTP::isalnum((char)*iter) && (!asUrl || skip.find((char)*iter) == std::string::npos)) {
           // bit more complex replace
           result << "%" << std::hex << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(*iter);
         } else result << (char)*iter;
@@ -314,6 +342,8 @@ namespace YaHTTP {
            return "Requested range not satisfiable";
        case 417:
            return "Expectation Failed";
+       case 422:
+           return "Unprocessable Entity";
        case 500:
            return "Internal Server Error";
        case 501:
@@ -331,9 +361,9 @@ namespace YaHTTP {
        }
     }; //<! static HTTP codes to text mappings
 
-    static std::map<std::string,std::string> parseUrlParameters(std::string parameters) {
+    static strstr_map_t parseUrlParameters(std::string parameters) {
       std::string::size_type pos = 0;
-      std::map<std::string,std::string> parameter_map;
+      strstr_map_t parameter_map;
       while (pos != std::string::npos) {
         // find next parameter start
         std::string::size_type nextpos = parameters.find("&", pos);
@@ -389,13 +419,25 @@ namespace YaHTTP {
       return iequals(a,b,a.size());
     }; //<! case-insensitive comparison
 
+    static void trimLeft(std::string &str) {
+       const std::locale &loc = std::locale::classic();
+       std::string::iterator iter = str.begin();
+       while(iter != str.end() && YaHTTP::isspace(*iter, loc)) iter++;
+       str.erase(str.begin(), iter);
+    }; //<! removes whitespace from left
+
     static void trimRight(std::string &str) {
        const std::locale &loc = std::locale::classic();
        std::string::reverse_iterator iter = str.rbegin();
-       while(iter != str.rend() && std::isspace(*iter, loc)) iter++;
+       while(iter != str.rend() && YaHTTP::isspace(*iter, loc)) iter++;
        str.erase(iter.base(), str.end());
     }; //<! removes whitespace from right
 
+    static void trim(std::string &str) {
+       trimLeft(str);
+       trimRight(str);
+    }; //<! removes whitespace from left and right
+
     static std::string camelizeHeader(const std::string &str) {
        std::string::const_iterator iter = str.begin();
        std::string result;
@@ -415,4 +457,3 @@ namespace YaHTTP {
    }; //<! camelizes headers, such as, content-type => Content-Type
   };
 };
-#endif
