@@ -66,10 +66,7 @@ static void initArguments(int argc, char** argv)
   
   cleanSlashes(configname);
 
-  if(!::arg().preParseFile(configname.c_str(), "socket-dir", ""))
-    cerr<<"Warning: unable to parse configuration file '"<<configname<<"'"<<endl;
-  if(!::arg().preParseFile(configname.c_str(), "chroot", ""))
-    cerr<<"Warning: unable to parse configuration file '"<<configname<<"'"<<endl;
+  arg().laxFile(configname.c_str());
 
   arg().laxParse(argc,argv);   // make sure the commandline wins
 
@@ -107,7 +104,7 @@ try
       command+=" ";
     command+=commands[i];
   }
-  rccS.send(command);
+  rccS.send(command, nullptr, arg().asNum("timeout"));
   string receive=rccS.recv(0, arg().asNum("timeout"));
   if(receive.compare(0, 7, "Unknown") == 0) {
     cerr<<receive<<endl;
