@@ -1,11 +1,10 @@
 #include <unistd.h>
+#include "threadname.hh"
 #include "remote_logger.hh"
-
 #include <sys/uio.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-
 #ifdef PDNS_CONFIG_ARGS
 #include "logger.hh"
 #define WE_ARE_RECURSOR
@@ -85,7 +84,7 @@ bool RemoteLogger::reconnect()
   }
   catch(const std::exception& e) {
 #ifdef WE_ARE_RECURSOR
-    L<<Logger::Warning<<"Error connecting to remote logger "<<d_remote.toStringWithPort()<<": "<<e.what()<<std::endl;
+    g_log<<Logger::Warning<<"Error connecting to remote logger "<<d_remote.toStringWithPort()<<": "<<e.what()<<std::endl;
 #else
     warnlog("Error connecting to remote logger %s: %s", d_remote.toStringWithPort(), e.what());
 #endif
@@ -115,9 +114,17 @@ void RemoteLogger::queueData(const std::string& data)
   }
 }
 
+
 void RemoteLogger::maintenanceThread()
 try
 {
+#ifdef WE_ARE_RECURSOR
+  string threadName = "pdns-r/remLog";
+#else
+  string threadName = "dnsdist/remLog";
+#endif
+  setThreadName(threadName);
+
   for(;;) {
     if(d_exiting)
       break;
